#!/bin/bash

# pid: 7510, remote: 2280, user: stateless_agent, host: horslan.ddbuiq.tech, local: 80 stateless_agent@horslan.ddbuiq.tech -p 22020, host: horslan.ddbuiq.tech

function printWithin () {
	rowSize=$1
	string=$2

	strSize=$(echo $string | wc -c)
	maxSize=$((rowSize-3))
	if [[ $strSize -lt $rowSize ]]; then
		# Ca rentre
		diffSize=$((rowSize-strSize))
		for ((i=1; i<=$diffSize; i++)); do
			string=$string' '
		done
		echo $string
	else
		# ca rentre pas
		echo "${string:0:$maxSize}... "
	fi
}

function getTunnels () {
	echo -e "+-----------------+--------------------------+-------+------------------+"
	echo -e "|      user       |       target host        |  pid  |      mapping     |"
	echo -e "+-----------------+--------------------------+-------+------------------+"

	pgrep -af autossh | while read -r pid cmd; do
		techStuff=$(echo $cmd | awk -F'NR ' '{print$2}')
		remotePort=$(echo $techStuff | awk -F':' '{print $1}')
		localPort=$(echo $techStuff | awk -F':' '{print $3}' | awk '{print $1}')
		user=$(echo $techStuff | awk -F'@' '{print $1}' | awk '{print $2}')
		host=$(echo $techStuff | awk -F'@' '{print $2}' | awk '{print $1}')

		#echo "pid: $pid, remote: $remotePort, user: $user, host: $host, local: $localPort, host: $host"
		echo -e "| ${user} | ${host} | ${pid} |  ${localPort}->${remotePort}\t|"
	done
	echo -e "+-----------------+--------------------------+-------+------------------+\n"
}

function setBandwidthValue() {
	Bts=$1

	if [[ $Bts -gt 1000000 ]]; then
		# > 1mb
		ret=$(echo $((Bts/1000000)).$((Bts%1000000/1024/100)))
		units='mb'

	elif [[ $Bts -gt 1000 ]]; then
		# > kb
		ret=$(echo $((Bts/1024)).$((Bts%1024/100)))
		units='kb'

	else
		# bytes
		ret=$Bts
		units='b'
	fi

	echo "${ret}${units}"
}

function humanReadableTime () {
	secs=$1

	printf '%02d:%02d:%02d\n' $((secs/3600)) $((secs%3600/60)) $((secs%60))
}

function getConnectedOVPNClients () {
	# Common Name,Real Address,Bytes Received,Bytes Sent,Connected Since
	# entosis-regw-1,127.0.0.1:47444,58821,60018,2023-11-06 11:25:29
	clientConnected=$(cat /var/log/openvpn/status.log | grep -B 3 'ROUTING TABLE' | grep -i 'entosis-regw-')

	if [[ $clientConnected != '' ]]; then
		clientName=$(echo $clientConnected | awk -F, '{print $1}')
		clientPort=$(echo $clientConnected | awk -F, '{print $2}' | awk -F: '{print $2}')
		clientRx=$(echo $clientConnected | awk -F, '{print $3}')
		clientTx=$(echo $clientConnected | awk -F, '{print $4}')
		clientConSince=$(echo $clientConnected | awk -F, '{print $5}')
		clientLastSeen=$(cat /var/log/openvpn/status.log | grep -A 3 'ROUTING TABLE' | grep -i $clientName | awk -F, '{print $4}')

		#echo "since: '${clientConSince}'"
		#echo "Seen: '${clientLastSeen}'"

		nowEpoch=$(date +"%s")
		clientConSinceEpoch=$(date -d "$clientConSince" +"%s")
		clientLastSeenEpoch=$(date -d "$clientLastSeen" +"%s")

		clientConSinceTime=$(humanReadableTime $((nowEpoch-clientConSinceEpoch)))
		clientLastSeenTime=$(humanReadableTime $((nowEpoch-clientLastSeenEpoch)))

		echo -e "+-----------------+--------+------+------+-----------------+-----------------+"
		echo -e "|      user       |  port  |  rx  |  tx  | connected since |    last seen    |"
		echo -e "+-----------------+--------+------+------+-----------------+-----------------+"
		echo -e "| $(printWithin 17 ${clientName})| $(printWithin 8 ${clientPort})| $(printWithin 6 $(setBandwidthValue ${clientRx}))| $(printWithin 6 $(setBandwidthValue ${clientTx}))| $(printWithin 17 ${clientConSinceTime})| $(printWithin 17 ${clientLastSeenTime})|"
		echo -e "+-----------------+--------+------+------+-----------------+-----------------+"
	fi
}

#printWithin 10 "toto"
getTunnels
getConnectedOVPNClients
